/*
 *
 * Copyright (C) D-Type Solutions - All Rights Reserved
 * Web Site: http://www.d-type.com
 * E-mail: info@d-type.com
 *
 * This file is part of D-Type Engine and may only be used, modified
 * and distributed under the terms of the applicable license agreement
 * For conditions of distribution and use, see license.txt
 *
 * ANY VIOLATIONS OF THE ABOVE LICENSE AGREEMENT WILL BE PROSECUTED TO
 * THE MAXIMUM EXTENT POSSIBLE UNDER THE LAW. NO VENDOR, DISTRIBUTOR,
 * DEALER, RETAILER, SALES PERSON OR OTHER PERSON IS AUTHORIZED TO MODIFY
 * THIS AGREEMENT OR TO MAKE ANY WARRANTY, REPRESENTATION OR PROMISE WHICH
 * IS DIFFERENT THAN, OR IN ADDITION TO, THIS AGREEMENT.
 *
 */


#ifndef DTYPE_DTYPE3DR_H
#define DTYPE_DTYPE3DR_H


#ifdef __cplusplus
DT_EXTERN {
#endif


/* ------------------------- D-Type 3D Rasterizer ------------------------- */

typedef float DT_R3DPOINT;


typedef union
{
	DT_UBYTE COLX[4]; /* color (BGRT) */
	DT_UWORD UV[2]; /* texture coord */

} DT_R3DCOLOR;


typedef struct
{
	DT_R3DPOINT X, Y, Z;
	DT_R3DCOLOR Color;

} DT_R3DVERTEX;


typedef struct
{
	DT_R3DVERTEX Vertices[3];
	DT_ULONG TextureID;
	DT_UBYTE MiscBits1; /* texture rendering quality and other bits */
	DT_UBYTE MiscBits2; /* reserved for future */
	DT_UBYTE COLX[4]; /* average triangle color (BGRT) */

} DT_R3DTRIANGLE;


typedef struct
{
	const DT_FLOAT* Pos;
	const DT_FLOAT* Rot;
	const DT_FLOAT* Mul;
	const DT_FLOAT* Arr; /* can be DV_NULL, reserved for future */
	const DT_UBYTE* Bkg; /* can be DV_NULL */
	const DT_UBYTE* Env; /* can be DV_NULL, reserved for future */
	const DT_R3DPOINT* Buff; /* can be DV_NULL */
	const DT_R3DPOINT* Proj; /* can be DV_NULL */
	DT_ULONG MiscBits; /* 0 = perspective, 1 = orthography */
	DT_UWORD MaxDepth;
	DT_UWORD Extra1; /* reserved for future */
	DT_UWORD Extra2; /* reserved for future */
	DT_UWORD Extra3; /* reserved for future */
	DT_SLONG Width;
	DT_SLONG Height;
	DT_UBYTE Rasterizer;
	DT_UBYTE Quality;
	DT_UBYTE Shading;
	DT_UBYTE Culling;
	DT_UBYTE Clipping;
	DT_UBYTE TransformMode;
	DT_UBYTE TranspThreshold;
	DT_UBYTE Attribs; /* reserved for future */

} DT_R3DVIEW;


typedef struct
{
	DT_ULONG Nodes_Count;
	DT_ULONG Nodes_Size;
	DT_ULONG Output_Count;
	DT_ULONG Output_Size;
	DT_ULONG Input_Count;
	DT_ULONG Input_Size;
	DT_UWORD Flags;
	DT_UBYTE Errors;
	DT_UBYTE Reserved;

} DT_R3DSTATS;


typedef struct
{
	DT_SLONG InitKey;    /* Initialization key (0=demo) */
	DT_SWORD InitFlag;   /* Initialization flag (0=demo) */
	DT_SWORD Reserved1;  /* Must be set to 0 */
	DT_SWORD Reserved2;  /* Must be set to 0 */

} DT_R3DINFO;


typedef struct DT_R3DMODEL_STRUCT* DT_R3DMODEL;
typedef struct DT_R3DRASTER_STRUCT* DT_R3DRASTER;


#define DV_R3DFORMAT_STL 0
#define DV_R3DFORMAT_AUTO 0
#define DV_R3DFORMAT_NO_COLORS 0
#define DV_R3DFORMAT_FACE_VERT_COLORS 1
#define DV_R3DFORMAT_ONLY_VERT_COLORS 2
#define DV_R3DFORMAT_ONLY_FACE_COLORS 3


#define DV_R3DADAPT_SHADE_FLAT 0
#define DV_R3DADAPT_SHADE_SMOOTH 1
#define DV_R3DADAPT_SHADE_SMOOTH_A 1
#define DV_R3DADAPT_SHADE_SMOOTH_B 2
#define DV_R3DADAPT_SHADE_SMOOTH_C 3
#define DV_R3DADAPT_TEXTUREQUALITY_COARSE_A 4
#define DV_R3DADAPT_TEXTUREQUALITY_SMOOTH_A 5
#define DV_R3DADAPT_TEXTUREQUALITY_COARSE_B 6
#define DV_R3DADAPT_TEXTUREQUALITY_SMOOTH_B 7
#define DV_R3DADAPT_CHROMA_SWAP_RB 8
#define DV_R3DADAPT_CHROMA_SET 9


#define DV_R3DTEXTURETYPE_STATIC 0
#define DV_R3DTEXTURETYPE_MIPMAP 1
#define DV_R3DTEXTURETYPE_HYBRID 2
#define DV_R3DTEXTURETYPE_DYNAMIC 3


#define DV_R3DTEXTUREFORMAT_1BPP_P 0
#define DV_R3DTEXTUREFORMAT_1BPP_M 1
#define DV_R3DTEXTUREFORMAT_2BPP_PX 2
#define DV_R3DTEXTUREFORMAT_3BPP_COL 3
#define DV_R3DTEXTUREFORMAT_4BPP_COLX 4


#define DV_R3DTEXTUREQUALITY_COARSE_A 0
#define DV_R3DTEXTUREQUALITY_SMOOTH_A 1
#define DV_R3DTEXTUREQUALITY_COARSE_B 2
#define DV_R3DTEXTUREQUALITY_SMOOTH_B 3


/* */
DT_SWORD DTYPE_APICALL drModelIni(DT_R3DMODEL* m3d, DT_R3DINFO* init_info);
/* */
void DTYPE_APICALL drModelErase(DT_R3DMODEL m3d);
/* */
void DTYPE_APICALL drModelExt(DT_R3DMODEL m3d);

/* */
DT_SWORD DTYPE_APICALL drModelSetTextureCallback(DT_R3DMODEL m3d, void (*user_texture_func)(DT_UBYTE* m, DT_ULONG texture_id, DT_ULONG user_id, DT_ULONG w, DT_ULONG h, DT_UBYTE texture_type, DT_UBYTE texture_format, DT_FLOAT texture_scale, void* user_param), void* user_param);
/* */
const DT_R3DTRIANGLE* DTYPE_APICALL drModelGetElement(DT_R3DMODEL m3d, DT_ULONG i);
/* */
DT_SWORD DTYPE_APICALL drModelAddElement(DT_R3DMODEL m3d, const DT_R3DTRIANGLE* element, DT_UWORD flags);
/* */
DT_SWORD DTYPE_APICALL drModelAddElements(DT_R3DMODEL m3d, DT_ULONG n, const DT_R3DTRIANGLE elements[], DT_UWORD flags);
/* */
DT_ULONG DTYPE_APICALL drModelAddTexture(DT_R3DMODEL m3d, DT_UBYTE flags, DT_UBYTE texture_type, DT_UBYTE texture_format, DT_UBYTE const_alpha, DT_ULONG w, DT_ULONG h, DT_ULONG user_id, const DT_UBYTE palette[]);
/* */
DT_SWORD DTYPE_APICALL drModelBuild(DT_R3DMODEL m3d, DT_UWORD flags);
/* */
void DTYPE_APICALL drModelTransform(DT_R3DMODEL m3d, const DT_TM4X4 transform);
/* */
DT_SWORD DTYPE_APICALL drModelAdapt(DT_R3DMODEL m3d, DT_ULONG bgn, DT_ULONG len, DT_UWORD method, DT_UWORD flags, const DT_R3DPOINT point[], const DT_UBYTE colx_a[], const DT_UBYTE colx_b[], const DT_FLOAT shade_val[]);
/* */
DT_ULONG drModelGetSize(DT_R3DMODEL m3d, DT_UWORD flags);
/* */
DT_ULONG DTYPE_APICALL drModelGetBounds(DT_R3DMODEL m3d, DT_UWORD flags, DT_R3DPOINT bounds[]);
/* */
const DT_R3DSTATS* DTYPE_APICALL drModelGetStats(DT_R3DMODEL m3d);
/* */
DT_SWORD DTYPE_APICALL drModelAppend(DT_R3DMODEL m3d, DT_UWORD flags, DT_R3DPOINT x_off, DT_R3DPOINT y_off, DT_R3DPOINT z_off, const DT_R3DMODEL m3d2);
/* */
DT_SWORD DTYPE_APICALL drModelAppendFromStream(DT_R3DMODEL m3d, DT_UWORD flags, DT_UWORD file_format, DT_R3DPOINT x_off, DT_R3DPOINT y_off, DT_R3DPOINT z_off, const DT_STREAM_DESC* sd);
/* */
DT_SWORD DTYPE_APICALL drModelSaveToStream(DT_R3DMODEL m3d, DT_UWORD flags, DT_UWORD file_format, DT_STREAM_DESC* sd);


/* */
DT_SWORD DTYPE_APICALL drRasterizerIni(DT_R3DRASTER* r3d, DT_R3DINFO* init_info);
/* */
DT_SWORD DTYPE_APICALL drRasterizerIniPlus(DT_R3DRASTER* r3d, DT_R3DINFO* init_info, DT_DASYS das, DT_UWORD flags);
/* */
void DTYPE_APICALL drRasterizerExt(DT_R3DRASTER r3d);

/* */
DT_SWORD DTYPE_APICALL drRasterizerSetMDC(DT_R3DRASTER r3d, DT_ID_SWORD format, DT_ID_SWORD subformat, const DT_MDC* memory_surface);
/* */
void DTYPE_APICALL drRasterizerSetASys(DT_R3DRASTER r3d, DT_DASYS das);
/* */
void DTYPE_APICALL drRasterizerGetASys(DT_R3DRASTER r3d, DT_DASYS* das);

/* */
DT_SWORD DTYPE_APICALL drRasterizerDoOutput(DT_R3DRASTER r3d, const DT_R3DMODEL model, const DT_R3DVIEW* view, const DT_TM3X3 tm, DT_UWORD flags);
/* */
DT_SWORD DTYPE_APICALL drRasterizerDoExport(DT_R3DRASTER r3d, const DT_R3DMODEL model, const DT_R3DVIEW* view, const DT_TM3X3 tm, DT_UWORD flags, DT_UWORD file_format, DT_STREAM_DESC* sd);

/* */
DT_SWORD DTYPE_APICALL drMakeImage(DT_R3DRASTER r3d, const DT_MDC* memory_surface, DT_UWORD flags, DT_UWORD file_format, DT_UBYTE quality, DT_UBYTE reserved, const DT_CHAR* filename);


#ifdef __cplusplus
}
#endif


#endif /* DTYPE_DTYPE3DR_H */
